package Cltest::ChangeInfo;
use 5.018;
use warnings;

use JSON::PP;
use Term::ANSIColor qw(colored);
use Tie::File;
use Cltest::PatchsetBuildResult;
use Cltest::Utils;
use File::Path qw(make_path);
use File::Basename qw(dirname);
use if $ENV{DEBUG}, 'Smart::Comments';

# return hash
sub get_change_detail
{
	my $ua = Cltest::Utils::get_ua();
	my $cl_num = shift;
	my $url = "https://cr.deepin.io/changes/$cl_num/detail";
	my $resp = $ua->get($url);
	if ($resp->is_success ){
		$_ = $resp->decoded_content;
		return decode_json( substr( $_, index ($_, "\n") + 1) );
	}
	else {
		die "get_change_detail : url = $url ",$resp->status_line;
	}
}

sub new {
	my ($class, $cl_num) = @_;
	my $detail = get_change_detail($cl_num);
	### $detail
	my $self = bless {
		project => $detail->{project},
		status => $detail->{status},
		subject => $detail->{subject},
		owner => $detail->{owner}{name},
		num => $detail->{_number},
	}, $class;

	my $messages = $detail->{messages};
	### $messages
	my @jenkins_messages;
	for (@$messages) {
		my $author_name = $_->{author}{name};
		if (defined $author_name && $author_name eq 'jenkins' ) {
			push @jenkins_messages, $_->{message};
		}
	}
	### @jenkins_messages
	my @build_results;
	for my $msg ( @jenkins_messages ) {
		if ( $msg =~ /Patch Set (\d+): Verified/ ) {
			### jenkins Verified: $msg
			my $patchset_num = $1;
			if ( my %url_status = $msg =~ m{(https?://\S+).+(SUCCESS|FAILURE)}g ) {
				### %url_status
				for my $url (keys %url_status) {
					next if $url =~ /-mxe-ci/;
					next if $url =~ /-win32-ci/;
					my $status = $url_status{ $url };
					push @build_results, Cltest::PatchsetBuildResult->new($patchset_num, $url, $status, $self);
				}
			}
		}
	}

	$self->{patchset_build_results} = \@build_results;
	return $self;
}

sub show_info
{
	my $self = shift;
	my @show_fields = qw(subject status owner);
	for my $key (@show_fields) {
		say colored( ucfirst $key . ': ' . $self->{$key}, 'yellow');
	}
}

our $history_file = "$ENV{HOME}/.cache/cltest/history.txt";
sub save_history {
	my $self = $_[0];
	make_path(dirname($history_file));
	tie my @history, 'Tie::File', $history_file
		or die "failed tie file `$history_file`: $!";

	my $subject_first_line = (split /\n/, $self->{subject})[0];
	my $line = sprintf "%s [%s] %s -- %s",
		$self->{num}, $self->{project}, $subject_first_line, $self->{owner};
	if ( @history >= 10 ) {
		pop @history;
	}
	unshift @history, $line;
	untie @history;
}

# TODO: 考虑接受个 patchset num 参数
sub install_deb {
	my $self = shift;
	$self->show_info;

	my $build_results = $self->{patchset_build_results};
	unless (@$build_results) {
		say "Not found any patchset build result.";
		return;
	}

	# get newest
	#warn "build results:", pp($build_results);
	my $max_patchset_num = 0;
	for my $br ( @$build_results ) {
		my $patchset_num = $br->{patchset_num};
		if ( $patchset_num > $max_patchset_num ) {
			$max_patchset_num = $patchset_num;
		}
	}

	my @newest_build_results =
		grep { $_->{patchset_num} == $max_patchset_num && $_->is_success  } @$build_results;

	my $selected_build;
	if ( @newest_build_results == 0 ) {
		warn "not found";
		return
	} elsif ( @newest_build_results == 1 ) {
		$selected_build = $newest_build_results[0];
	} else {
		# > 1
		my $reply_idx = Cltest::Utils::get_reply(
			prompt => "Please select build url: ",
			choices => [map { $_->{url} } @newest_build_results ],
		);
		$selected_build = $newest_build_results[ $reply_idx ];
	}

	$selected_build->install_debs;
	$self->save_history;
}


1;
