package Cltest::Utils;
use warnings;
use strict;
use 5.018;

use LWP::UserAgent;
use Term::ANSIColor qw(colored);

our $ua = LWP::UserAgent->new(
	ssl_opts => { verify_hostname => 0 },
	show_progress => 1,
);

sub get_ua {
    return $ua;
}

sub uniq (@)
{
    my %seen = ();
    my $k;
    my $seen_undef;
    grep { defined $_ ? not $seen{ $k = $_ }++ : not $seen_undef++ } @_;
}

sub ask_yn {
    my ($prompt, $default) = @_;
    if ($default eq 'y') {
        $prompt .= " (Yes/n) ";
    } elsif ( $default eq 'n' ) {
        $prompt .= " (y/No) ";
    } else {
        die "default must be y or n";
    }

    print colored($prompt, 'green');
    my $line = <STDIN>;
    chomp $line;
    if ($line =~ /^\s*$/ ) {
        # $line is empty
        $line = $default;
    }
    return $line =~ /^y/i;
}

sub get_reply {
	my %arg = @_;
	my $prompt = $arg{prompt};
	my $choices = $arg{choices};

	print "\n";
	for my $i (keys @$choices ) {
		say "[$i] $choices->[$i]";
	}
	print "\n",$prompt;

	my $max_idx = @$choices - 1;
	if ($max_idx <= 0) {
		warn "No other choice";
		return 0;
	}
	my $idx;
	while (1) {
		my $line = <STDIN>;
		chomp $line;
		if ( $line =~ /^\d+$/ ) {
			my $num = int($line);
			if ($num <= $max_idx) {
				$idx = $num;
				last;
			}
		}
		print "[0~$max_idx] " . $prompt;
	}
	return $idx;
}

1;
